/*
 * 	SERVER0.C
 *	ESEMPIO DI SCRITTURA DI UN SERVER
 *      ECHO SERVER con le seguenti caratteristiche:
 *      - possibilita` di servire un client per volta
 *      - orientato alla linea
 *      - chiusura di una connessione alla ricezione della linea "close"
 *      - shutdown del server alla ricezione della linea "stop"
 */


#include    <stdio.h>
#include    <stdlib.h>
#include    <string.h>
#include    "mysocket.h"

#define RBUFLEN		128

/* Prototipi delle funzioni */
int getline (char* , char*);
void hit_return(void);
int writen(SOCKET, char *, int);
int readline (SOCKET s, char *ptr, int maxlen);

void main()
{
	SOCKET	conn_request_skt;	/* socket connections are received on */
	char	buf[RBUFLEN];		/* receiver buffer */
	unsigned short lport_n, lport_h;/* port connections are accepted on */
	int	bklog = 2;		/* listen backlog */
	SOCKET		     s;
	int		     result, addrlen, n;
	struct sockaddr_in   saddr, c_addr;

	/* Inizializzazione interfaccia dei socket */
	SockStartup();

	/* input del numero di porta utilizzato dal server */
	getline(buf, "Enter port : ");
	lport_h = atoi(buf);
	lport_n = htons(lport_h);

	/* creazione del socket */
	printf("creating socket\n");
	s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s == INVALID_SOCKET)
		err_fatal("socket() failed");
	printf("done, socket number %u\n",s);

	/* bind */
	saddr.sin_family      = AF_INET;
	saddr.sin_port        = lport_n;
	saddr.sin_addr.s_addr = INADDR_ANY;
	showAddr("Binding to address",&saddr);
	result = bind(s, (struct sockaddr *) &saddr, sizeof(saddr));
	if (result == -1)
		err_fatal("bind() failed");
	printf("done.\n");

	/* listen */
	printf ("Listening at socket %d with backlog = %d \n",s,bklog);
	result = listen(s, bklog);
	if (result == -1)
		err_fatal("listen() failed");
	printf("done.\n");

	conn_request_skt = s;

	/* ciclo principale */
	for (;;)
	{
		addrlen = sizeof(struct sockaddr_in);
		s = accept(conn_request_skt, (struct sockaddr *) &c_addr, &addrlen);
		if (s == INVALID_SOCKET)
			err_fatal("accept() failed");
		showAddr("Accepted connection from", &c_addr);
		printf("new socket: %u\n",s);

		for (;;)
		{
			n=readline(s, buf, RBUFLEN-1);
	        	if (n <= 0)
			{
		    	    printf("Read error/Connection closed\n");
			    closesocket(s);
			    printf("Socket %d closed", s);
			    break;
			}
	        	else
			{   printf("Received line from socket %03d :\n", s);
			    
			    printf("[%s]\n",buf);
			    if((strcmp(buf,"data\n\0"))==0)
			           {buf[0]='3';
			            buf[1]='5';
		  	            buf[2]='4';
			            buf[3]='\0';}			
			    if(writen(s, buf, n) != n)
			    	printf("Write error while replying\n");
			    else
			    	printf("Reply sent\n");
			}
			if (n < 1 || strcmp(buf, "close\n") == 0)
			{
				closesocket(s);
				printf("Socket %d closed", s);
				break;
			}
			else if (strcmp(buf, "stop\n") == 0)
			{
				closesocket(s);
				printf("Socket %d closed\n", s);
				closesocket(conn_request_skt);
				printf("Socket %d closed\n", conn_request_skt);
				SockCleanup();
				exit(0);
			}
				
		}
	}
}


/* Funzione per l'input di una linea di testo con stringa di invito */
int getline(char *line, char *prompt)
{
	char	ch;

	printf("%s", prompt);
	while ((ch = getchar()) != '\n' && ch != EOF)
		*line++ = ch;
	*line = '\0';
	if (ch == EOF)
		return(EOF);
	else    return(1);
}

/* Read a line from stream socket s to buffer ptr 
   store it (including newline) in buffer ptr
   whose length is maxlen */
int readline (SOCKET s, char *ptr, int maxlen)
{
    int n, nread;
    char c;

    for (n=1; n<maxlen; n++)
    {
	nread=recv(s, &c, 1, 0);
	if (nread == 1)
	{
	    *ptr++ = c;
	    if (c == '\n') break;
	}
	else if (nread == 0)	/* connection closed by party */
	{
	    if (n == 1)
		return (0);
	    else 
	    	break;
	}
	else 			/* error */
	    return (-1);
    }
    *ptr = 0;
    return (n);
}


/* Write nbytes from buffer ptr to stream socket s */
int writen(SOCKET s, char *ptr, int nbytes)
{
    int nleft, nwritten;

    for (nleft=nbytes; nleft > 0; )
    {
	nwritten = send(s, ptr, nleft, 0);
	if (nwritten <=0)
	    return (nwritten);
	else
	{
	    nleft -= nwritten;
	    ptr += nwritten;   
	}
    }
    return (nbytes - nleft);
}
